<?php

namespace Unific\Connector\Helper\Data;

use Magento\Framework\Exception\InputException;
use Symfony\Component\Config\Definition\Exception\Exception;

class Customer extends \Magento\Framework\App\Helper\AbstractHelper
{
    protected $filterHelper;

    /**
     * @var \Magento\Framework\Api\ExtensibleDataInterface
     */
    protected $guestBillingAddress;

    /**
     * @var \Magento\Framework\Api\ExtensibleDataInterface
     */
    protected $guestShippingAddress;

    protected $customerSession;
    protected $customerFactory;
    protected $addressFactory;
    protected $addressCollectionFactory;

    // Holds a customer API DATA object
    protected $customer;

    private $dataFormatter;

    protected $returnData = [
        'entity_id' => 0,
        'customer_is_guest' => 1,
        'email' => '',
        'prefix' => '',
        'firstname' => '',
        'middlename' => '',
        'lastname' => '',
        'suffix' => '',
        'dob' => '',
        'gender' => '',
        'created_at' => '',
        'updated_at' => '',
        'addresses' => []
    ];

    /**
     * OrderPlugin constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Unific\Connector\Helper\Filter $filterHelper
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Customer\Model\CustomerFactory $customerFactory
     * @param \Magento\Customer\Model\AddressFactory $addressFactory
     * @param \Magento\Customer\Model\ResourceModel\Address\CollectionFactory $addressCollectionFactory
     * @param Formatter $dataFormatter
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Unific\Connector\Helper\Filter $filterHelper,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Customer\Model\CustomerFactory $customerFactory,
        \Magento\Customer\Model\AddressFactory $addressFactory,
        \Magento\Customer\Model\ResourceModel\Address\CollectionFactory $addressCollectionFactory,
        Formatter $dataFormatter
    ) {
        parent::__construct($context);

        $this->filterHelper = $filterHelper;

        $this->customerSession = $customerSession;
        $this->customerFactory = $customerFactory;
        $this->addressFactory = $addressFactory;
        $this->addressCollectionFactory = $addressCollectionFactory;
        $this->dataFormatter = $dataFormatter;
    }

    /**
     * @param \Magento\Customer\Api\Data\CustomerInterface $customer
     */
    public function setCustomer(\Magento\Customer\Api\Data\CustomerInterface $customer)
    {
        $this->customer = $customer;
        $this->setCustomerInfo();
    }

    /**
     * @return \Magento\Customer\Model\Data\Customer
     */
    public function getCustomer()
    {
        if ($this->customer == null && $this->customerSession->isLoggedIn() === true) {
            $this->setCustomer($this->customerSession->getCustomer()->getDataModel());
        }

        return $this->customer;
    }

    /**
     *
     */
    public function generateGuestCustomer($entity)
    {
        $customer = $this->customerFactory->create();

        // Using the billing address because the item might not need shipping
        if ($entity->getBillingAddress() != null) {
            $customer->setEmail($entity->getBillingAddress()->getEmail());
            $customer->setPrefix($entity->getBillingAddress()->getPrefix());
            $customer->setFirstname($entity->getBillingAddress()->getFirstname());
            $customer->setMiddlename($entity->getBillingAddress()->getMiddlename());
            $customer->setLastname($entity->getBillingAddress()->getLastname());
            $customer->setSuffix($entity->getBillingAddress()->getSuffix());
            $customer->setDob($entity->getBillingAddress()->getDob());
            $customer->setGender($entity->getBillingAddress()->getGender());

            $this->guestBillingAddress = ($entity->getBillingAddress()) ? $entity->getBillingAddress() : $entity->getShippingAddress();
            $this->guestShippingAddress = ($entity->getShippingAddress()) ? $entity->getShippingAddress() : $entity->getBillingAddress();
        }

        $this->setCustomer($customer->getDataModel());
    }

    /**
     * @return mixed
     */
    protected function setCustomerInfo()
    {
        if ($this->customer->getId() !== null) {
            $this->returnData['entity_id'] = $this->customer->getId();
            $this->returnData['customer_id'] = $this->customer->getId();
        }

        $this->returnData['customer_is_guest'] = (int)($this->customer->getId() === null);
        $this->returnData['email'] = $this->customer->getEmail();
        $this->returnData['prefix'] = $this->customer->getPrefix();
        $this->returnData['firstname'] = $this->customer->getFirstname();
        $this->returnData['middlename'] = $this->customer->getMiddlename();
        $this->returnData['lastname'] = $this->customer->getLastname();
        $this->returnData['suffix'] = $this->customer->getSuffix();
        $this->returnData['dob'] = $this->customer->getDob();
        $this->returnData['gender'] = $this->customer->getGender();
        $this->returnData['created_at'] = $this->customer->getCreatedAt();
        $this->returnData['updated_at'] = $this->customer->getUpdatedAt();

        if ($this->returnData['created_at'] == null) {
            $this->returnData['created_at'] = date('Y-m-d H:i:s');
        }

        if ($this->returnData['updated_at'] == null) {
            $this->returnData['updated_at'] = date('Y-m-d H:i:s');
        }

        if ($this->customer->getId() === null) {
            $this->setGuestAddresses();
        } else {
            $this->setCustomerAddresses();
        }

        if(isset($this->returnData['addresses']) && count($this->returnData['addresses']) == 0) {
            unset($this->returnData['addresses']);
        }
    }

    /**
     * @return array
     */
    public function getCustomerInfo()
    {
        return $this->filterHelper->sanitizeAddressData($this->returnData);
    }

    /**
     * Set the addresses based on guest data
     */
    public function setGuestAddresses()
    {
        if ($this->guestBillingAddress) {
            $this->returnData['addresses']['billing'] = $this->guestBillingAddress->getData();
            $this->returnData['addresses']['billing']['street'] = $this->guestBillingAddress->getStreetFull();
            $this->returnData['addresses']['billing'] = $this->dataFormatter->setStreetData(
                $this->returnData['addresses']['billing'],
                $this->guestBillingAddress->getStreetFull()
            );
        }

        if ($this->guestShippingAddress) {
            $this->returnData['addresses']['shipping'] = $this->guestShippingAddress->getData();
            $this->returnData['addresses']['shipping']['street'] = $this->guestBillingAddress->getStreetFull();

            $this->returnData['addresses']['shipping'] = $this->dataFormatter->setStreetData(
                $this->returnData['addresses']['shipping'],
                $this->guestShippingAddress->getStreetFull()
            );

        }
    }

    /**
     * Set the address of a logged in customer
     */
    protected function setCustomerAddresses()
    {
        $billingAddressId = $this->customer->getDefaultBilling();
        $shippingAddressId = $this->customer->getDefaultShipping();

        //billing
        if ($billingAddressId) {
            $billingAddress = $this->addressFactory->create()->load($billingAddressId);
            $this->returnData['addresses']['billing'] = $billingAddress->getData();
            $this->returnData['addresses']['billing']['street'] = $billingAddress->getStreetFull();

            $this->returnData['addresses']['billing'] = $this->dataFormatter->setStreetData(
                $this->returnData['addresses']['billing'],
                $billingAddress->getStreetFull()
            );


            if ($shippingAddressId === null) {
                $this->returnData['addresses']['shipping'] = $billingAddress->getData();
                $this->returnData['addresses']['shipping']['street'] = $billingAddress->getStreetFull();
                $this->returnData['addresses']['shipping'] = $this->dataFormatter->setStreetData(
                    $this->returnData['addresses']['shipping'],
                    $billingAddress->getStreetFull()
                );
            }
        }

        if ($shippingAddressId) {
            $shippingAddress = $this->addressFactory->create()->load($shippingAddressId);
            $this->returnData['addresses']['shipping'] = $shippingAddress->getData();
            $this->returnData['addresses']['shipping']['street'] = $shippingAddress->getStreetFull();

            $this->returnData['addresses']['shipping'] = $this->dataFormatter->setStreetData(
                $this->returnData['addresses']['shipping'],
                $shippingAddress->getStreetFull()
            );

            if ($billingAddressId === null) {
                $this->returnData['addresses']['billing'] = $shippingAddress->getData();
                $this->returnData['addresses']['billing']['street'] = $shippingAddress->getStreetFull();

                $this->returnData['addresses']['billing'] = $this->dataFormatter->setStreetData(
                    $this->returnData['addresses']['billing'],
                    $shippingAddress->getStreetFull()
                );

            }
        }
    }
}
