<?php

namespace Unific\Connector\Helper;

use Magento\Framework\Exception\InputException;
use Symfony\Component\Config\Definition\Exception\Exception;

class Queue extends \Magento\Framework\App\Helper\AbstractHelper
{
    protected $logger;
    protected $restConnection;
    protected $queueCollectionFactory;

    protected $hmacHelper;
    protected $productMetadata;
    protected $moduleList;

    /**
     * OrderPlugin constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param Logger $logger
     * @param Hmac $hmacHelper
     * @param \Unific\Connector\Model\ResourceModel\Queue\CollectionFactory $queueCollectionFactory
     * @param \Unific\Connector\Connection\Connection $restConnection
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        Logger $logger,
        Hmac $hmacHelper,
        \Unific\Connector\Model\ResourceModel\Queue\CollectionFactory $queueCollectionFactory,
        \Unific\Connector\Connection\Connection $restConnection,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\Module\ModuleListInterface $moduleList
    ) {
        parent::__construct($context);

        $this->logger = $logger;
        $this->restConnection = $restConnection;
        $this->queueCollectionFactory = $queueCollectionFactory;

        $this->hmacHelper = $hmacHelper;
        $this->productMetadata = $productMetadata;
        $this->moduleList = $moduleList;
    }

    /**
     * @return void
     */
    public function process()
    {
        $this->sendDataFromQueue(false, Settings::QUEUE_LIVE_PER_MINUTE);
        $this->sendDataFromQueue(true, Settings::QUEUE_HISTORICAL_PER_MINUTE);
    }

    /**
     * @param bool $isHistorical
     * @param int $size
     */
    protected function sendDataFromQueue($isHistorical = false, $size = 100)
    {
        $collection = $this->queueCollectionFactory->create();
        $collection->addFieldToFilter('historical', ['eq', (int)$isHistorical]);
        //$collection->addFieldToFilter('retry_amount', ['lt', Settings::QUEUE_MAX_RETRIES]);
        $collection->setPageSize($size);
        $collection->setOrder('priority', 'ASC');
        $collection->setCurPage(1);

        // For efficiency we do not loop everything constantly
        $headers = array();
        $headers['Content-Type'] = 'application/json';
        $headers['Accept'] = 'application/json';
        $headers['X-UNIFIC-INTEGRATION-ID'] = $this->scopeConfig->getValue('unific/connector/integration', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $headers['X-MAGENTO-VERSION'] = $this->productMetadata->getVersion();
        $headers['X-UNIFIC-CONNECTOR-VERSION'] = $this->moduleList->getOne('Unific_Connector')['setup_version'];

        foreach ($collection as $queueItem) {
            switch ($queueItem->getRequestType()) {
                case 'POST':
                    $type = \Zend\Http\Request::METHOD_POST;
                    break;
                case 'PUT':
                    $type = \Zend\Http\Request::METHOD_PUT;
                    break;
                case 'DELETE':
                    $type = \Zend\Http\Request::METHOD_DELETE;
                    break;
                default:
                    $type = \Zend\Http\Request::METHOD_GET;
                    break;
            }

            $headers = array_merge($headers, json_decode($queueItem->getHeaders(), true));
            $headers['X-MAGENTO-UNIFIC-HMAC'] = $this->hmacHelper->generateHmac(json_decode($queueItem->getMessage(), true));
            $headers['X-MAGENTO-GUID'] = $queueItem->getGuid();

            $response = $this->restConnection->sendData($queueItem->getUrl(), json_decode($queueItem->getMessage(), true), $headers, $type);

            if($response->isSuccess()) {
                $this->logger->createLog($queueItem, $response, 'Webhook sent succesfully and removed from queue');
                $queueItem->delete();
            } else {
                $queueItem->setRetryAmount($queueItem->getRetryAmount() + 1);
                $queueItem->setResponseHttpCode($response->getStatusCode());
                $queueItem->save();

                if($queueItem->getRetryAmount() == $queueItem->getMaxRetryAmount()) {
                    $this->logger->createLog($queueItem, $response, 'Webhook reached maximum retry amount. Sending is stopped until a retry is forced.');
                } else {
                    $this->logger->createLog($queueItem, $response, 'Webhook failed, queueing retry attempt ' . $queueItem->getRetryAmount() . '/' . $queueItem->getMaxRetryAmount());
                }
            }
        }
    }
}
